@@ -1,17 +1,33 @@
 from __future__ import print_function
+import sys
 import multiprocessing
 import warnings
 import dill
 from tqdm.auto import tqdm
 from traceback import format_exc
-from pickle import PicklingError
+from pickle import PicklingError, dispatch_table
 
-from pickle import _Pickler, dispatch_table
+PY3 = (sys.hexversion >= 0x3000000)
 
+try:
+    from cStringIO import StringIO
+except ImportError:
+    if PY3:
+        from io import BytesIO as StringIO
+    else:
+        from StringIO import StringIO
 
-class Pickler(_Pickler):
+failed_rv = (lambda *args, **kwargs: None, ())
+
+class Pickler(dill.Pickler):
+    """ Overload dill to ignore unpickleble parts of objects.
+        You probably didn't want to use these parts anyhow.
+        However, if you did, you'll have to find some way to make them pickleble.
+    """
     def save(self, obj, save_persistent_id=True):
-        self.framer.commit_frame()
+        """ Copied from pickle and amended. """
+        if PY3:
+            self.framer.commit_frame()
 
         # Check for persistent id (defined by a subclass)
         pid = self.persistent_id(obj)
@@ -63,11 +79,14 @@ class Pickler(_Pickler):
                             raise PicklingError("Can't pickle %r object: %r" %
                                                 (t.__name__, obj))
                 except:
-                    rv = (lambda *args, **kwargs: None, ())
+                    rv = failed_rv
 
         # Check for string returned by reduce(), meaning "save as global"
         if isinstance(rv, str):
-            self.save_global(obj, rv)
+            try:
+                self.save_global(obj, rv)
+            except:
+                self.save_global(obj, failed_rv)
             return
 
         # Assert that reduce() returned a tuple
@@ -81,11 +100,19 @@ class Pickler(_Pickler):
                                 "two to six elements" % reduce)
 
         # Save the reduce() output and finally memoize the object
-        self.save_reduce(obj=obj, *rv)
-
-dill.StockPickler = Pickler
-
+        try:
+            self.save_reduce(obj=obj, *rv)
+        except:
+            self.save_reduce(obj=obj, *failed_rv)
 
+def dumps(obj, protocol=None, byref=None, fmode=None, recurse=None, **kwds):#, strictio=None):
+    """pickle an object to a string"""
+    protocol = dill.settings['protocol'] if protocol is None else int(protocol)
+    _kwds = kwds.copy()
+    _kwds.update(dict(byref=byref, fmode=fmode, recurse=recurse))
+    file = StringIO()
+    Pickler(file, protocol, **_kwds).dump(obj)
+    return file.getvalue()
 
 def chunks(n, *args):
     """ Yield successive n-sized chunks from lists. """
@@ -248,7 +275,7 @@ class parpool(object):
 
     @fun.setter
     def fun(self, fun):
-        funs = dill.dumps(fun, recurse=True)
+        funs = dumps(fun, recurse=True)
         self._fun = (fun, hash(funs), funs)
 
     @property
@@ -257,7 +284,7 @@ class parpool(object):
 
     @args.setter
     def args(self, args):
-        argss = dill.dumps(args, recurse=True)
+        argss = dumps(args, recurse=True)
         self._args = (args, hash(argss), argss)
 
     @property
@@ -266,7 +293,7 @@ class parpool(object):
 
     @kwargs.setter
     def kwargs(self, kwargs):
-        kwargss = dill.dumps(kwargs, recurse=True)
+        kwargss = dumps(kwargs, recurse=True)
         self._kwargs = (kwargs, hash(kwargss), kwargss)
 
     def __enter__(self, *args, **kwargs):
@@ -288,8 +315,8 @@ class parpool(object):
                         .format(r[1]))
                 warnings.formatwarning = pfmt
                 fun, args, kwargs = [dill.loads(f[1]) for f in r[2][1:]]
-                r = (False, r[1], fun(r[2][0], *args, **kwargs))
-            self.res[r[1]] = r[2]
+                r = (False, r[1], fun(dill.loads(r[2][0]), *args, **kwargs))
+            self.res[r[1]] = dill.loads(r[2])
             if not self.bar is None:
                 self.bar.update()
             self._qbar_update()
@@ -298,6 +325,7 @@ class parpool(object):
 
     def __call__(self, n, fun=None, args=None, kwargs=None, handle=None):
         """ Add new iteration, using optional manually defined handle."""
+        n = dumps(n, recurse=True)
         if not fun is None:
             self.fun = fun
         if not args is None:
@@ -388,7 +416,7 @@ class parpool(object):
                     fun = self.get_from_cache(*Fun)
                     args = self.get_from_cache(*Args)
                     kwargs = self.get_from_cache(*Kwargs)
-                    self.Qo.put((False, i, fun(n, *args, **kwargs)))
+                    self.Qo.put((False, i, dumps(fun(dill.loads(n), *args, **kwargs), recurse=True)))
                 except multiprocessing.queues.Empty:
                     continue
                 except:
